rm(list=ls())
require(keras)
require(tensorflow)
require(PerfMeas)

# generate input and outcome data (these are randomly generated for the sole purpose of testing the code)
x <- data.frame(matrix(rnorm(1000000*20), ncol = 20), gest=rnorm(200000, mean=38*7, sd=18))
x[sample(nrow(x),80000),"gest"] <- 999              # simulate 80000 cases with unknown gestational age (i.e. gest=999)
y <- data.frame(matrix(rbinom(1000000*12, size=1, prob=0.1), ncol=12))
names(y) <- c('rds','ivh','nec','rop','bpd','pda','pvl','sepsis','pulmhem','cp','pulmhtn','jaundice')                   

# input and outcome matrices
y=as.matrix(y)
x=as.matrix(x)

##split train, validation and test
set.seed(1)
index=sample(nrow(x), 200000)
train=index
test=seq(nrow(x))[-train]
xtrain=(x[index,])
ytrain=(y[index,])
xtest=x[-index,]
ytest=y[-index,]

set.seed(99)
val_index = sample(nrow(xtest), 100000)
xval=(xtest[val_index,])
yval=(ytest[val_index,])
xtest=xtest[-val_index,]
ytest=ytest[-val_index,]

### create variable to stratify into GA-specific groups
gtrain <- cut(xtrain[,"gest"], breaks=c(0,32*7,37*7,40*7,50*7,10000), labels=1:5,right=F)
gtest <- cut(xtest[,"gest"], breaks=c(0,32*7,37*7,40*7,50*7,10000), labels=1:5,right=F)
gval <- cut(xval[,"gest"], breaks=c(0,32*7,37*7,40*7,50*7,10000), labels=1:5,right=F)

# remove gestational age from the input dataset
xtrain <- xtrain[,!colnames(xtrain) %in% c("gest")]
xval <- xval[,!colnames(xval) %in% c("gest")]
xtest <- xtest[,!colnames(xtest) %in% c("gest")]

# build the deep NN multi-task model
input <- layer_input(shape=ncol(xtrain))

base.model <- input %>%
  layer_masking(mask_value=9999) %>%
  layer_batch_normalization() %>%
  layer_dense(units = 16, activation = 'tanh', input_shape = ncol(xtrain)) %>% 
  layer_dense(units = 32, activation = 'relu') %>%
  layer_dense(units = 32, activation = 'tanh') %>%
  layer_batch_normalization() %>%
  layer_dropout(rate = 0.3)

# add outputs
PULMHTN <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="PULMHTN")

JAUNDICE <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="JAUNDICE")

SEPSIS <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="SEPSIS")

PDA <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="PDA")

RDS <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="RDS")

ROP <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="ROP")

BPD <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="BPD")

IVH <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="IVH")

NEC <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="NEC")

PULMHEM <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="PULMHEM")

PVL <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="PVL")

CP <- base.model %>% 
  layer_dense(units = 16) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 8) %>%
  layer_dropout(rate = 0.3) %>% 
  layer_dense(units = 1, activation='sigmoid', name="CP")

base.model1 <- keras_model(input,list(PULMHTN,JAUNDICE,SEPSIS,PDA,RDS,ROP,BPD,IVH,NEC,PULMHEM,PVL,CP)) %>%
  compile(loss = "binary_crossentropy",optimizer = 'adam')

history1 <- base.model1 %>% fit(x = xtrain[gtrain==1,], y = list(PULMHTN=ytrain[gtrain==1,'pulmhtn'], JAUNDICE=ytrain[gtrain==1,'jaundice'], SEPSIS=ytrain[gtrain==1,'sepsis'], 
                                                                 PDA=ytrain[gtrain==1,'pda'], RDS=ytrain[gtrain==1,'rds'], ROP=ytrain[gtrain==1,'rop'], 
                                                                 BPD=ytrain[gtrain==1,'bpd'], IVH=ytrain[gtrain==1,'ivh'], NEC=ytrain[gtrain==1,'nec'], PULMHEM=ytrain[gtrain==1,'pulmhem'], 
                                                                 PVL=ytrain[gtrain==1,'pvl'], CP=ytrain[gtrain==1,'cp']), 
                                epochs = 200, batch_size = 512, verbose=2,
                                validation_data = list(x_val = xval[gval==1,],y_val = list(PULMHTN=yval[gval==1,"pulmhtn"], JAUNDICE=yval[gval==1,"jaundice"], SEPSIS=yval[gval==1,"sepsis"], 
                                                                                           PDA=yval[gval==1,"pda"], RDS=yval[gval==1,"rds"], ROP=yval[gval==1,"rop"], BPD=yval[gval==1,"bpd"], 
                                                                                           IVH=yval[gval==1,"ivh"], NEC=yval[gval==1,"nec"], PULMHEM=yval[gval==1,"pulmhem"],PVL=yval[gval==1,"pvl"], CP=yval[gval==1,"cp"])), 
                                callbacks=list(callback_early_stopping(monitor='val_loss', verbose=1, patience=10),
                                               callback_model_checkpoint(filepath='best_model_oct20_1_nogaall.h5', monitor='val_loss', save_best_only=T))) 


base.model2 <- keras_model(input,list(PULMHTN,JAUNDICE,SEPSIS,PDA,RDS,ROP,BPD,IVH,NEC,PULMHEM,PVL,CP)) %>%
  compile(loss = "binary_crossentropy",optimizer = 'adam')

history2 <- base.model2 %>% fit(x = xtrain[gtrain==2,], y = list(PULMHTN=ytrain[gtrain==2,'pulmhtn'], JAUNDICE=ytrain[gtrain==2,'jaundice'], SEPSIS=ytrain[gtrain==2,'sepsis'], 
                                                                 PDA=ytrain[gtrain==2,'pda'], RDS=ytrain[gtrain==2,'rds'], ROP=ytrain[gtrain==2,'rop'], 
                                                                 BPD=ytrain[gtrain==2,'bpd'], IVH=ytrain[gtrain==2,'ivh'], NEC=ytrain[gtrain==2,'nec'], PULMHEM=ytrain[gtrain==2,'pulmhem'], 
                                                                 PVL=ytrain[gtrain==2,'pvl'], CP=ytrain[gtrain==2,'cp']), 
                                epochs = 100, batch_size = 512, verbose=2,
                                validation_data = list(x_val = xval[gval==2,],y_val = list(PULMHTN=yval[gval==2,"pulmhtn"], JAUNDICE=yval[gval==2,"jaundice"], SEPSIS=yval[gval==2,"sepsis"], 
                                                                                           PDA=yval[gval==2,"pda"], RDS=yval[gval==2,"rds"], ROP=yval[gval==2,"rop"], BPD=yval[gval==2,"bpd"], 
                                                                                           IVH=yval[gval==2,"ivh"], NEC=yval[gval==2,"nec"], PULMHEM=yval[gval==2,"pulmhem"],PVL=yval[gval==2,"pvl"], CP=yval[gval==2,"cp"])), 
                                callbacks=list(callback_early_stopping(monitor='val_loss', verbose=1, patience=10),
                                               callback_model_checkpoint(filepath='best_model_oct20_2_nogaall.h5', monitor='val_loss', save_best_only=T))) 


base.model3 <- keras_model(input,list(PULMHTN,JAUNDICE,SEPSIS,PDA,RDS,ROP,BPD,IVH,NEC,PULMHEM,PVL,CP)) %>%
  compile(loss = "binary_crossentropy",optimizer = 'adam')

history3 <- base.model3 %>% fit(x = xtrain[gtrain==3,], y = list(PULMHTN=ytrain[gtrain==3,'pulmhtn'], JAUNDICE=ytrain[gtrain==3,'jaundice'], SEPSIS=ytrain[gtrain==3,'sepsis'], 
                                                                 PDA=ytrain[gtrain==3,'pda'], RDS=ytrain[gtrain==3,'rds'], ROP=ytrain[gtrain==3,'rop'], 
                                                                 BPD=ytrain[gtrain==3,'bpd'], IVH=ytrain[gtrain==3,'ivh'], NEC=ytrain[gtrain==3,'nec'], PULMHEM=ytrain[gtrain==3,'pulmhem'], 
                                                                 PVL=ytrain[gtrain==3,'pvl'], CP=ytrain[gtrain==3,'cp']), 
                                epochs = 100, batch_size = 512, verbose=2,
                                validation_data = list(x_val = xval[gval==3,],y_val = list(PULMHTN=yval[gval==3,"pulmhtn"], JAUNDICE=yval[gval==3,"jaundice"], SEPSIS=yval[gval==3,"sepsis"], 
                                                                                           PDA=yval[gval==3,"pda"], RDS=yval[gval==3,"rds"], ROP=yval[gval==3,"rop"], BPD=yval[gval==3,"bpd"], 
                                                                                           IVH=yval[gval==3,"ivh"], NEC=yval[gval==3,"nec"], PULMHEM=yval[gval==3,"pulmhem"],PVL=yval[gval==3,"pvl"], CP=yval[gval==3,"cp"])), 
                                callbacks=list(callback_early_stopping(monitor='val_loss', verbose=1, patience=10),
                                               callback_model_checkpoint(filepath='best_model_oct20_3_nogaall.h5', monitor='val_loss', save_best_only=T))) 


base.model4 <- keras_model(input,list(PULMHTN,JAUNDICE,SEPSIS,PDA,RDS,ROP,BPD,IVH,NEC,PULMHEM,PVL,CP)) %>%
  compile(loss = "binary_crossentropy",optimizer = 'adam')

history4 <- base.model4 %>% fit(x = xtrain[gtrain==4,], y = list(PULMHTN=ytrain[gtrain==4,'pulmhtn'], JAUNDICE=ytrain[gtrain==4,'jaundice'], SEPSIS=ytrain[gtrain==4,'sepsis'], 
                                                                 PDA=ytrain[gtrain==4,'pda'], RDS=ytrain[gtrain==4,'rds'], ROP=ytrain[gtrain==4,'rop'], 
                                                                 BPD=ytrain[gtrain==4,'bpd'], IVH=ytrain[gtrain==4,'ivh'], NEC=ytrain[gtrain==4,'nec'], PULMHEM=ytrain[gtrain==4,'pulmhem'], 
                                                                 PVL=ytrain[gtrain==4,'pvl'], CP=ytrain[gtrain==4,'cp']), 
                                epochs = 100, batch_size = 512, verbose=2,
                                validation_data = list(x_val = xval[gval==4,],y_val = list(PULMHTN=yval[gval==4,"pulmhtn"], JAUNDICE=yval[gval==4,"jaundice"], SEPSIS=yval[gval==4,"sepsis"], 
                                                                                           PDA=yval[gval==4,"pda"], RDS=yval[gval==4,"rds"], ROP=yval[gval==4,"rop"], BPD=yval[gval==4,"bpd"], 
                                                                                           IVH=yval[gval==4,"ivh"], NEC=yval[gval==4,"nec"], PULMHEM=yval[gval==4,"pulmhem"],PVL=yval[gval==4,"pvl"], CP=yval[gval==4,"cp"])), 
                                callbacks=list(callback_early_stopping(monitor='val_loss', verbose=1, patience=10),
                                               callback_model_checkpoint(filepath='best_model_oct20_4_nogaall.h5', monitor='val_loss', save_best_only=T))) 


base.model5 <- keras_model(input,list(PULMHTN,JAUNDICE,SEPSIS,PDA,RDS,ROP,BPD,IVH,NEC,PULMHEM,PVL,CP)) %>%
  compile(loss = "binary_crossentropy",optimizer = 'adam')

history5 <- base.model5 %>% fit(x = xtrain[gtrain==5,], y = list(PULMHTN=ytrain[gtrain==5,'pulmhtn'], JAUNDICE=ytrain[gtrain==5,'jaundice'], SEPSIS=ytrain[gtrain==5,'sepsis'], 
                                                                 PDA=ytrain[gtrain==5,'pda'], RDS=ytrain[gtrain==5,'rds'], ROP=ytrain[gtrain==5,'rop'], 
                                                                 BPD=ytrain[gtrain==5,'bpd'], IVH=ytrain[gtrain==5,'ivh'], NEC=ytrain[gtrain==5,'nec'], PULMHEM=ytrain[gtrain==5,'pulmhem'], 
                                                                 PVL=ytrain[gtrain==5,'pvl'], CP=ytrain[gtrain==5,'cp']), 
                                epochs = 100, batch_size = 512, verbose=2,
                                validation_data = list(x_val = xval[gval==5,],y_val = list(PULMHTN=yval[gval==5,"pulmhtn"], JAUNDICE=yval[gval==5,"jaundice"], SEPSIS=yval[gval==5,"sepsis"], 
                                                                                           PDA=yval[gval==5,"pda"], RDS=yval[gval==5,"rds"], ROP=yval[gval==5,"rop"], BPD=yval[gval==5,"bpd"], 
                                                                                           IVH=yval[gval==5,"ivh"], NEC=yval[gval==5,"nec"], PULMHEM=yval[gval==5,"pulmhem"],PVL=yval[gval==5,"pvl"], CP=yval[gval==5,"cp"])), 
                                callbacks=list(callback_early_stopping(monitor='val_loss', verbose=1, patience=10),
                                               callback_model_checkpoint(filepath='best_model_oct20_5_nogaall.h5', monitor='val_loss', save_best_only=T))) 


bestmodel1 <- load_model_hdf5('best_model_oct20_1_nogaall.h5',compile = F)
bestmodel2 <- load_model_hdf5('best_model_oct20_2_nogaall.h5',compile = F)
bestmodel3 <- load_model_hdf5('best_model_oct20_3_nogaall.h5',compile = F)
bestmodel4 <- load_model_hdf5('best_model_oct20_4_nogaall.h5',compile = F)
bestmodel5 <- load_model_hdf5('best_model_oct20_5_nogaall.h5',compile = F)

##make predictions
testpredictor1 <- bestmodel1 %>% predict(xtest)
testpredictor2 <- bestmodel2 %>% predict(xtest)
testpredictor3 <- bestmodel3 %>% predict(xtest)
testpredictor4 <- bestmodel4 %>% predict(xtest)
testpredictor5 <- bestmodel5 %>% predict(xtest)

## assign prediction from the relevan GA-specific group
test_stackedpred <- data.frame(id=1:nrow(xtest))

for(jj in 1:12) {
  test_stackedpred[gtest==1,jj+1] <- testpredictor1[[jj]][gtest==1]
  test_stackedpred[gtest==2,jj+1] <- testpredictor2[[jj]][gtest==2]
  test_stackedpred[gtest==3,jj+1] <- testpredictor3[[jj]][gtest==3]
  test_stackedpred[gtest==4,jj+1] <- testpredictor4[[jj]][gtest==4]
  test_stackedpred[gtest==5,jj+1] <- testpredictor5[[jj]][gtest==5]   
}

names(test_stackedpred) <- c("id", "pulmhtn", "jaundice", "sepsis", "pda", "rds", "rop", "bpd", "ivh", "nec", "pulmhem", "pvl", "cp")

##calculate AUPRC and AUC in the test data
outcomes <- c("pulmhtn", "jaundice", "sepsis", "pda", "rds", "rop", "bpd", "ivh", "nec", "pulmhem", "pvl", "cp")
testres=vector()
testAUC=vector()
for (out in outcomes) {
  z=precision.at.all.recall.levels(test_stackedpred[,out], ytest[,out])
  testres[which(outcomes==out)]=(trap.rule.integral(z$recall, z$precision))
  testAUC[which(outcomes==out)]=AUC.single(test_stackedpred[,out], ytest[,out])
}

# AUPR
data.frame(o=outcomes, testres)

# AUC
data.frame(o=outcomes, testAUC)

rm(list = ls())



